# -*- coding: utf-8 -*-
"""Returns and calculates financial data.

AnalyzeCompany
    GenericData
    FinancialData
        FinancialPeriod
            IncomeStatement
            BalanceSheet
            Ratios
"""

from .const import (
    ATTR_COMPANY_NAME,
    ATTR_COMPANY_COUNTRY,
    ATTR_FINANCIAL_EQUITY,
    ATTR_REPORT_DATE,
    BALANCE_SHEET_ITEMS,
    CASH_AND_OTHER_EQUIVALENTS,
    FINANCIAL_DATA,
    ATTR_FORMAL_NAME,
    GENERIC_INFORMATION,
    INCOME_STATEMENT_ITEMS,
    INTEREST_BEARING_DEBT,
    INCOME_STATEMENT_OPERATIVE_INCOME,
    INCOME_STATEMENT_COGS,
    INCOME_STATEMENT_ADMIN_COSTS,
    INCOME_STATEMENT_DEPRECIATION,
    INCOME_STATEMENT_AMORTIZATION
)


class AnalyzeCompany():
    """Initialize base object."""

    def __init__(self, raw):
        """Init class."""
        self.raw = raw

    @property
    def generic_data(self):
        """Get data on name, country etc."""
        return GenericData(self)

    @property
    def financial_data(self):
        """Get financial data (balance sheet, ratios etc)."""
        return FinancialData(self)


class GenericData(AnalyzeCompany):
    """Everything related to a specific company."""

    def __init__(self, generic_data):
        """Init class."""
        self._generic_data = generic_data

    @property
    def raw(self):
        """Return raw data that it represents."""
        return self._generic_data.raw[GENERIC_INFORMATION]

    @property
    def name(self):
        """Get name of company."""
        return self.raw.get(ATTR_COMPANY_NAME)

    @property
    def country(self):
        """Get country of company."""
        return self.raw.get(ATTR_COMPANY_COUNTRY)


class FinancialData(AnalyzeCompany):
    """A specific company's financial data."""

    def __init__(self, financial_data):
        """Init class."""
        self._financial_data = financial_data

    @property
    def raw(self):
        """Return raw data that it represents."""
        return self._financial_data.raw[FINANCIAL_DATA]

    @property
    def period(self):
        """Return light objects of the light control."""
        return [FinancialPeriod(self.raw, i) for i in range(len(self.raw))]


class FinancialPeriod():
    """Represent a financial period, eg a quarter."""

    def __init__(self, period, index):
        self.period = period
        self.index = index

    @property
    def raw(self):
        """Return raw data that it represents."""
        return self.period[self.index]

    @property
    def generic_information(self):
        """Return raw data that it represents."""
        return FinancialPeriodGeneric(self)

    @property
    def balance_sheet(self):
        """Get balance sheet object."""
        return BalanceSheet(self)

    @property
    def income_statement(self):
        """Get income statement object."""
        return IncomeStatement(self)

    @property
    def ratios(self):
        """Get ratios object."""
        return Ratios(self)


class FinancialPeriodGeneric(FinancialPeriod):
    """Generic information about the period."""

    def __init__(self, generic_information):
        """Init class."""
        self._generic_information = generic_information

    @property
    def raw(self):
        """Return raw data that it represents."""
        return self._generic_information.raw[GENERIC_INFORMATION]

    @property
    def report_date(self):
        """Return report date."""
        return self.raw.get(ATTR_REPORT_DATE)

    @property
    def formal_name(self):
        """Return report date."""
        return self.raw.get(ATTR_FORMAL_NAME)


class BalanceSheet(FinancialPeriod):
    """A specific company's balance sheet."""

    def __init__(self, balance_sheet):
        """Init class."""
        self._balance_sheet = balance_sheet

    @property
    def raw(self):
        """Return raw data that it represents."""
        return self._balance_sheet.raw[BALANCE_SHEET_ITEMS]

    @property
    def equity(self):
        """Get shareholder's equity."""
        return self.raw.get(ATTR_FINANCIAL_EQUITY)

    @property
    def net_debt(self):
        """Get net debt."""
        return (self.raw.get(INTEREST_BEARING_DEBT) -
                self.raw.get(CASH_AND_OTHER_EQUIVALENTS))


class IncomeStatement(FinancialData):
    """A specific company's income statement."""

    def __init__(self, income_statement):
        """Init class."""
        self._income_statement = income_statement

    @property
    def raw(self):
        """Return raw data that it represents."""
        return self._income_statement.raw[INCOME_STATEMENT_ITEMS]

    @property
    def ebitda(self):
        """Return defined income row EBITDA."""
        return (self.raw.get(INCOME_STATEMENT_OPERATIVE_INCOME) -
                self.raw.get(INCOME_STATEMENT_COGS) -
                self.raw.get(INCOME_STATEMENT_ADMIN_COSTS) -
                self.raw.get(INCOME_STATEMENT_DEPRECIATION) -
                self.raw.get(INCOME_STATEMENT_AMORTIZATION))


class Ratios(FinancialData):
    """Calculate all ratios that interest us."""

    @property
    def income_statement(self):
        """Get income statement data from parent."""
        return self._financial_data.income_statement

    @property
    def balance_sheet(self):
        """Get balance sheet data from parent."""
        return self._financial_data.balance_sheet

    @property
    def gross_margin(self):
        """Calculate gross margin."""
        return (self.income_statement.raw.get(INCOME_STATEMENT_COGS) /
                self.income_statement.raw.get(
                    INCOME_STATEMENT_OPERATIVE_INCOME)
                )

    @property
    def net_debt_ebitda(self):
        """Calculate net debt to ebitda."""
        return self.balance_sheet.net_debt / self.income_statement.ebitda

    @property
    def net_debt_equity(self):
        """Calculate net debt to equity."""
        return self.balance_sheet.net_debt / self.balance_sheet.equity
